@@ -1,4 +1,4 @@
-# Copyright (C) 2007 - 2008 LottaNZB Development Team
+# Copyright (C) 2007-2010 LottaNZB Development Team
 # 
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -13,6 +13,7 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA.
 
+import traceback
 import xmlrpclib
 import socket
 import errno
@@ -20,24 +21,43 @@ import errno
 import logging
 log = logging.getLogger(__name__)
 
+from xml.dom.minidom import parse
 from time import sleep
+from shutil import copy
 from os import remove
+from os.path import basename
 
 from kiwi.python import clamp
 from kiwi.utils import gsignal
 
+from lottanzb import resources
 from lottanzb.core import App
 from lottanzb.util import Thread, GObject, gproperty, _
-from lottanzb.config import GConfigSection
+from lottanzb.config import ConfigSection
 
-class Config(GConfigSection):
+class Config(ConfigSection):
     update_interval = gproperty(type=float, default=1.0, minimum=1.0)
+    
+    delete_enqueued_nzb_files = gproperty(
+        type    = bool,
+        default = False,
+        nick    = "Delete NZB files added to the download queue",
+        blurb   = "Setting this value to True doesn't mean that the NZB "
+                  "file is completely removed from the system, but that it "
+                  "can only be found in the HellaNZB's QUEUE_DIR."
+                  "Because it's very uncommon for files to disappear as"
+                  "soon as a user clicks on them, this option is set to "
+                  "False by default.")
 
 class Backend(Thread):
     updating = gproperty(type=bool, default=False)
     paused = gproperty(type=bool, default=False)
     connected = gproperty(type=bool, default=False)
     
+    address = gproperty(type=str)
+    port = gproperty(type=int)
+    password = gproperty(type=str)
+    
     gsignal("updated")
     gsignal("unexpected-disconnect")
     
@@ -118,7 +138,7 @@ class Backend(Thread):
                 raise ConnectionError, _("Could not connect to %s. This might be due to network issues.") % (address)
         
         except AssertionError, e:
-            raise UnsupportedDaemonError, _("You don't seem to be running a compatible HellaNZB daemon. Plase make sure you have installed the latest version.")
+            raise UnsupportedDaemonError, _("You don't seem to be running a compatible HellaNZB daemon. Please make sure you have installed the latest version.")
         
         else:
             setattr(proxy, "resume", getattr(proxy, "continue"))
@@ -134,6 +154,11 @@ class Backend(Thread):
             raise
         else:
             self.connected = True
+            
+            self.address = address
+            self.port = port
+            self.password = password
+            
             log.info(_("Connected to the HellaNZB daemon at %s.") % (location))
     
     def setProxy(self, proxy):
@@ -150,14 +175,89 @@ class Backend(Thread):
     def checkConnection(self, address, port, password):
         return self._get_server_proxy(address, port, password, True)
     
-    def enqueue(self, name):
+    def enqueue(self, file_name, new_file_name=""):
+        """
+        Adds an NZB file to the download queue.
+        
+        If LottaNZB is connected to a HellaNZB daemon running on the local
+        machine, we can directly pass the filename to it. Otherwise, the
+        whole content of the NZB file has to be transferred using XMLRPC,
+        which may take some time.
+        
+        If `new_file_name` is specified, the NZB file will be renamed to
+        the value of this argument. Please note that the file extension
+        must already be part of it.
+        
+        If the configuration option `backend.delete_enqueue_nzb_files` is set
+        to `True` the NZB file will be removed from the disk after it has
+        been added to the download queue.
+        """
+        
+        base_file_name = basename(file_name)
+        
+        # Workaround for a HellaNZB bug. See
+        #  - https://bugs.launchpad.net/ubuntu/+source/hellanzb/+bug/381318
+        #  - http://hellanzb.com/trac/hellanzb/ticket/425
+        # Postprocessing hangs if an NZB file with exactly the same name is
+        # embedded in the NZB file. Therefore, remove that entry from the NZB
+        # file to prevent it from being downloaded at all.
+        if not new_file_name or base_file_name == new_file_name:
+            try:
+                dom = parse(file_name)
+                nzb_file_removed = False
+                
+                for file_element in dom.getElementsByTagName("file"):
+                    if file_element.hasAttribute("subject") and \
+                        base_file_name in file_element.getAttribute("subject"):
+                        dom.documentElement.removeChild(file_element)
+                        nzb_file_removed = True
+                
+                if nzb_file_removed:
+                    dom.writexml(open(file_name, "w"))
+                    
+                    # Translators: An NZB file may tell a Usenet client like
+                    # HellaNZB to download any type of files, including other
+                    # NZB files. That's what is meant by "embedded" NZB file.
+                    # In order to work around a bug in HellaNZB, LottaNZB
+                    # sometimes changes NZB files so that the "embedded" NZB
+                    # files won't be downloaded by HellaNZB.
+                    log.info(_("Embedded NZB file removed from %s to avoid "
+                        "download issues.") % base_file_name)
+            except:
+                # Safety precaution. We don't want the `enqueue` method to fail
+                # just because of this workaround.
+                traceback.print_exc()
+        
         try:
-            self.update_state(self.__server.enqueue(name))
-            remove(name)
+            if self.address in ("localhost", "127.0.0.1"):
+                if new_file_name and base_file_name != new_file_name:
+                    temp_file_name = resources.get_user_temp(new_file_name)
+                    
+                    # Create a temporary copy of the NZB file with the new
+                    # name.
+                    copy(file_name, temp_file_name)
+                    
+                    self.update_state(self.__server.enqueue(temp_file_name))
+                    
+                    # Delete the temporary file.
+                    remove(temp_file_name)
+                else:
+                    self.update_state(self.__server.enqueue(file_name))
+            else:
+                nzb_data = open(file_name, "r").read()
+                
+                if new_file_name:
+                    file_name = new_file_name
+                
+                self.update_state(self.__server.enqueue(file_name, nzb_data))
         except:
             log.error(_("Could not add file to the download queue."))
         else:
-            log.info(_("Added %s to the download queue.") % (name))
+            log.info(_("Added %s to the download queue.") % (file_name))
+            
+            if self.config.delete_enqueued_nzb_files:
+                log.debug(_("Removing enqueued NZB file %s..."), file_name)
+                remove(file_name)
     
     def newzbinEnqueue(self, newzbin_id):
         try:
@@ -226,25 +326,31 @@ class Backend(Thread):
             log.info(_("Resuming downloads."))
     
     def move(self, download, target=None):
+        if not download.movable:
+            if download.state is Download.State.FINISHED:
+                log.info(_("Finished downloads cannot be moved."))
+            elif download.state is Download.State.PROCESSING:
+                log.info(_("Downloads that are being processed cannot be "
+                   "moved."))
+            
+            return
+        
         incomplete = self.downloads.get_incomplete()
         
-        try:
-            assert download.movable()
-            
-            if isinstance(target, Download):
-                if target in incomplete:
-                    target_index = incomplete.index(target)
-                else:
-                    target_index = 0
-            elif type(target) is int:
-                target_index = clamp(target, 0, len(incomplete) - 1)
-            elif not target:
-                target_index = len(incomplete) - 1
-            
-            assert download is not incomplete[target_index]
-        except:
-            return
-        else:
+        if isinstance(target, Download):
+            if target in incomplete:
+                target_index = incomplete.index(target)
+            else:
+                target_index = 0
+        elif type(target) is int:
+            target_index = clamp(target, 0, len(incomplete) - 1)
+        elif target is None:
+            # Move the download at the bottom of the queue
+            target_index = len(incomplete) - 1
+        
+        # If the following expression is `False`, the download is already
+        # at the right place.
+        if download is not incomplete[target_index]:
             if download is self.downloads.get_active_download():
                 self.__server.force(incomplete[1].id)
                 
@@ -254,8 +360,8 @@ class Backend(Thread):
                 self.__server.force(download.id)
             else:
                 self.__server.move(download.id, target_index)
-            
-            self.update_state()
+                
+                self.update_state()
     
     def move_up(self, download, shift=1):
         self.move_down(download, shift * -1)
@@ -309,17 +415,15 @@ class Backend(Thread):
             self.eta = state["eta"]
             self.remaining = state["queued_mb"]
             self.progress = state["percent_complete"]
-            
             self.log_entries = state["log_entries"]
             
             new_download_list = QueueList()
             
-            def update_download_list(download_type):
-                infos = state[download_type]
-                download_state = getattr(Download.State, download_type.upper())
-                
-                if "currently_downloading" or self.cache.get(download_type, None) != infos:
-                    for info in infos:
+            def update_download_list(download_state, download_infos):
+                if download_state is Download.State.DOWNLOADING or \
+                    self.cache.get(download_state, None) != download_infos:
+                    
+                    for info in download_infos:
                         download = self.downloads.get_by_id(info["id"])
                         
                         if download:
@@ -327,9 +431,13 @@ class Backend(Thread):
                         else:
                             download = Download(info, download_state)
                         
+                        if download_state is Download.State.DOWNLOADING:
+                            download.progress = self.progress
+                            download.remaining = self.remaining
+                        
                         new_download_list.append(download)
                     
-                    self.cache[download_type] = infos
+                    self.cache[download_state] = download_infos
                 else:
                     new_download_list.extend(self.downloads.get_by_state(download_state))
             
@@ -338,13 +446,14 @@ class Backend(Thread):
                     download.state == Download.State.DOWNLOADING or \
                     download.state == Download.State.PROCESSING)
             
-            state.update({
-                "processing": state["currently_processing"],
-                "downloading": state["currently_downloading"]
-            })
+            download_infos_by_state = [
+                (Download.State.PROCESSING, state["currently_processing"]),
+                (Download.State.DOWNLOADING, state["currently_downloading"]),
+                (Download.State.QUEUED, state["queued"])
+            ]
             
-            for download_type in ["processing", "downloading", "queued"]:
-                update_download_list(download_type)
+            for download_state, download_infos in download_infos_by_state:
+                update_download_list(download_state, download_infos)
             
             for download in self.downloads:
                 if not download in new_download_list and ( \
@@ -444,16 +553,16 @@ class QueueList(list):
         return 0
 
 class Download(GObject):
-    id = gproperty(type = int)
-    name = gproperty(type = str)
-    nzb_name = gproperty(type = str)
-    category = gproperty(type = str)
-    remaining = gproperty(type = int, minimum = 0)
-    progress = gproperty(type = int, minimum = 0, maximum = 100)
-    recovery = gproperty(type = bool, default = False)
-    password = gproperty(type = str)
-    size = gproperty(type = int, minimum = 0)
-    state = gproperty(type = int)
+    id = gproperty(type=int)
+    name = gproperty(type=str)
+    nzb_name = gproperty(type=str)
+    category = gproperty(type=str)
+    remaining = gproperty(type=int, minimum=0)
+    progress = gproperty(type=int)
+    recovery = gproperty(type=bool, default=False)
+    password = gproperty(type=str)
+    size = gproperty(type=int, minimum=0)
+    state = gproperty(type=int)
     
     def __init__(self, hellaInfo, state):
         GObject.__init__(self)
@@ -486,6 +595,22 @@ class Download(GObject):
         
         self.state = state
     
+    def set_property(self, key, value):
+        """
+        Make sure that the progress of a download is always between 0% and 100%.
+        
+        HellaNZB sporadically reports a progress of 101%, especially when
+        dealing with small downloads.
+        """
+        
+        if key == "progress":
+            if value > 100:
+                value = 100
+            elif value < 0:
+                value = 0
+        
+        GObject.set_property(self, key, value)
+    
     def dequeue(self):
         App().backend.dequeue(self)
     
@@ -502,19 +627,20 @@ class Download(GObject):
         if self.state == Download.State.PROCESSING or self.state == Download.State.FINISHED:
             self.progress = 100
             self.remaining = 0
-        elif self.state == Download.State.DOWNLOADING:
-            self.progress = App().backend.progress
-            self.remaining = App().backend.remaining
     
     def _handleSizeChange(self, *args):
         if self.size != 0 and not self.progress and not self.remaining:
             self.remaining = self.size
     
+    @property
     def movable(self):
-        return not (self.state == Download.State.FINISHED or self.state == Download.State.PROCESSING)
+        return not (
+            self.state == Download.State.FINISHED or
+            self.state == Download.State.PROCESSING
+        )
     
     def __str__(self):
         return self.name
     
     class State(object):
-        QUEUED, DOWNLOADING, PROCESSING, FINISHED = range(4)
\ No newline at end of file
+        QUEUED, DOWNLOADING, PROCESSING, FINISHED = range(4)
